@@ -1,4 +1,4 @@
-
+#
 # Copyright (C) 2012, Red Hat, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,13 +18,12 @@
 Middleware for authenticating against custom backends.
 """
 
-import logging
-
-from heat.openstack.common.gettextutils import _
-from heat.openstack.common import local
-from heat.rpc import client as rpc_client
+from oslo_context import context
+from oslo_log import log as logging
 import webob.exc
 
+from heat.common.i18n import _
+from heat.rpc import client as rpc_client
 
 LOG = logging.getLogger(__name__)
 
@@ -42,9 +41,9 @@ class AuthProtocol(object):
         Authenticate send downstream on success. Reject request if
         we can't authenticate.
         """
-        LOG.debug(_('Authenticating user token'))
-        context = local.store.context
-        authenticated = self.rpc_client.authenticated_to_backend(context)
+        LOG.debug('Authenticating user token')
+        ctx = context.get_current()
+        authenticated = self.rpc_client.authenticated_to_backend(ctx)
         if authenticated:
             return self.app(env, start_response)
         else:
