@@ -11,13 +11,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo.config import cfg
-from oslo.utils import importutils
-from stevedore import extension
 import warnings
 
-from heat.common.i18n import _
-from heat.openstack.common import log as logging
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import importutils
+import six
+from stevedore import extension
+
+from heat.common import exception
+from heat.common.i18n import _LE
+from heat.common.i18n import _LW
 
 LOG = logging.getLogger(__name__)
 
@@ -65,7 +69,7 @@ class OpenStackClients(object):
             client = getattr(self, method_name)()
             self._clients[name] = client
             return client
-        LOG.warn(_('Requested client "%s" not found') % name)
+        LOG.warn(_LW('Requested client "%s" not found'), name)
 
     @property
     def auth_token(self):
@@ -132,10 +136,14 @@ class ClientBackend(object):
         if cfg.CONF.cloud_backend == _default_backend:
             return OpenStackClients(context)
         else:
-            return importutils.import_object(
-                cfg.CONF.cloud_backend,
-                context
-            )
+            try:
+                return importutils.import_object(cfg.CONF.cloud_backend,
+                                                 context)
+            except (ImportError, RuntimeError) as err:
+                msg = _LE('Invalid cloud_backend setting in heat.conf '
+                          'detected  - %s') % six.text_type(err)
+                LOG.error(msg)
+                raise exception.Invalid(reason=msg)
 
 
 Clients = ClientBackend
@@ -155,5 +163,8 @@ def initialise():
 
     _mgr = extension.ExtensionManager(
         namespace='heat.clients',
-        invoke_on_load=False,
-        verify_requirements=False)
+        invoke_on_load=False)
+
+
+def list_opts():
+    yield None, cloud_opts
