@@ -137,14 +137,11 @@ class Resource(object):
             # Call is already for a subclass, so pass it through
             ResourceClass = cls
         else:
-            from heat.engine.resources import template_resource
-
             registry = stack.env.registry
-            try:
-                ResourceClass = registry.get_class(definition.resource_type,
-                                                   resource_name=name)
-            except exception.NotFound:
-                ResourceClass = template_resource.TemplateResource
+            ResourceClass = registry.get_class_to_instantiate(
+                definition.resource_type,
+                resource_name=name)
+
             assert issubclass(ResourceClass, Resource)
 
         return super(Resource, cls).__new__(ResourceClass)
@@ -309,6 +306,19 @@ class Resource(object):
         # common resources have not nested, StackResource overrides it
         return False
 
+    def resource_class(self):
+        """Return the resource class.
+
+        This is used to compare old and new resources when updating, to ensure
+        that in-place updates are possible. This method shold return the
+        highest common class in the hierarchy whose subclasses are all capable
+        of converting to each other's types via handle_update().
+
+        This mechanism may disappear again in future, so third-party resource
+        types should not rely on it.
+        """
+        return type(self)
+
     def has_hook(self, hook):
         # Clear the cache to make sure the data is up to date:
         self._data = None
@@ -331,7 +341,7 @@ class Resource(object):
             return True
         ri = self.stack.env.get_resource_info(self.type(),
                                               self.name)
-        return ri.name == resource_type
+        return ri is not None and ri.name == resource_type
 
     def implementation_signature(self):
         '''
