@@ -12,7 +12,6 @@
 #    under the License.
 
 import collections
-import copy
 import functools
 
 import six
@@ -23,20 +22,18 @@ from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.openstack.heat import software_config as sc
 from heat.engine.resources.openstack.heat import software_deployment as sd
-from heat.engine import rsrc_defn
 from heat.engine import support
 
 
 class StructuredConfig(sc.SoftwareConfig):
-    """A resource which has same logic with OS::Heat::SoftwareConfig.
-
+    '''
     This resource is like OS::Heat::SoftwareConfig except that the config
     property is represented by a Map rather than a String.
 
     This is useful for configuration tools which use YAML or JSON as their
     configuration syntax. The resulting configuration is transferred,
     stored and returned by the software_configs API as parsed JSON.
-    """
+    '''
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -68,8 +65,7 @@ class StructuredConfig(sc.SoftwareConfig):
 
 
 class StructuredDeployment(sd.SoftwareDeployment):
-    """A resource which has same logic with OS::Heat::SoftwareDeployment.
-
+    '''
     A deployment resource like OS::Heat::SoftwareDeployment, but which
     performs input value substitution on the config defined by a
     OS::Heat::StructuredConfig resource.
@@ -82,7 +78,7 @@ class StructuredDeployment(sd.SoftwareDeployment):
     with the value of input_name in this resource's input_values. If get_input
     needs to be passed through to the substituted configuration then a
     different input_key property value can be specified.
-    """
+    '''
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -117,7 +113,7 @@ class StructuredDeployment(sd.SoftwareDeployment):
         NAME: _sd_ps[NAME],
         INPUT_KEY: properties.Schema(
             properties.Schema.STRING,
-            _('Name of key to use for substituting inputs during deployment.'),
+            _('Name of key to use for substituting inputs during deployment'),
             default='get_input',
         ),
         INPUT_VALUES_VALIDATE: properties.Schema(
@@ -139,10 +135,10 @@ class StructuredDeployment(sd.SoftwareDeployment):
     def _build_derived_config(self, action, source,
                               derived_inputs, derived_options):
         cfg = source.get(sc.SoftwareConfig.CONFIG)
-        input_key = self.properties[self.INPUT_KEY]
-        check_input_val = self.properties[self.INPUT_VALUES_VALIDATE]
+        input_key = self.properties.get(self.INPUT_KEY)
+        check_input_val = self.properties.get(self.INPUT_VALUES_VALIDATE)
 
-        inputs = dict(i.input_data() for i in derived_inputs)
+        inputs = dict((i['name'], i['value']) for i in derived_inputs)
 
         return self.parse(inputs, input_key, cfg, check_input_val)
 
@@ -183,12 +179,7 @@ class StructuredDeployment(sd.SoftwareDeployment):
             return snippet
 
 
-class StructuredDeploymentGroup(sd.SoftwareDeploymentGroup):
-    """This resource associates a group of servers with some configuration.
-
-    This resource works similar as OS::Heat::SoftwareDeploymentGroup, but for
-    structured resources.
-    """
+class StructuredDeployments(sd.SoftwareDeployments):
 
     PROPERTIES = (
         SERVERS,
@@ -200,17 +191,17 @@ class StructuredDeploymentGroup(sd.SoftwareDeploymentGroup):
         INPUT_KEY,
         INPUT_VALUES_VALIDATE,
     ) = (
-        sd.SoftwareDeploymentGroup.SERVERS,
-        sd.SoftwareDeploymentGroup.CONFIG,
-        sd.SoftwareDeploymentGroup.INPUT_VALUES,
-        sd.SoftwareDeploymentGroup.DEPLOY_ACTIONS,
-        sd.SoftwareDeploymentGroup.NAME,
-        sd.SoftwareDeploymentGroup.SIGNAL_TRANSPORT,
+        sd.SoftwareDeployments.SERVERS,
+        sd.SoftwareDeployments.CONFIG,
+        sd.SoftwareDeployments.INPUT_VALUES,
+        sd.SoftwareDeployments.DEPLOY_ACTIONS,
+        sd.SoftwareDeployments.NAME,
+        sd.SoftwareDeployments.SIGNAL_TRANSPORT,
         StructuredDeployment.INPUT_KEY,
         StructuredDeployment.INPUT_VALUES_VALIDATE
     )
 
-    _sds_ps = sd.SoftwareDeploymentGroup.properties_schema
+    _sds_ps = sd.SoftwareDeployments.properties_schema
 
     properties_schema = {
         SERVERS: _sds_ps[SERVERS],
@@ -224,32 +215,25 @@ class StructuredDeploymentGroup(sd.SoftwareDeploymentGroup):
         StructuredDeployment.properties_schema[INPUT_VALUES_VALIDATE],
     }
 
-    def build_resource_definition(self, res_name, res_defn):
-        props = copy.deepcopy(res_defn)
-        servers = props.pop(self.SERVERS)
-        props[StructuredDeployment.SERVER] = servers.get(res_name)
-        return rsrc_defn.ResourceDefinition(res_name,
-                                            'OS::Heat::StructuredDeployment',
-                                            props, None)
-
-
-class StructuredDeployments(StructuredDeploymentGroup):
-
-    hidden_msg = _('Please use OS::Heat::StructuredDeploymentGroup instead.')
-    support_status = support.SupportStatus(
-        status=support.HIDDEN,
-        message=hidden_msg,
-        version='7.0.0',
-        previous_status=support.SupportStatus(
-            status=support.DEPRECATED,
-            version='2014.2'),
-        substitute_class=StructuredDeploymentGroup)
+    def _build_resource_definition(self, include_all=False):
+        p = self.properties
+        return {
+            self.RESOURCE_DEF_TYPE: 'OS::Heat::StructuredDeployment',
+            self.RESOURCE_DEF_PROPERTIES: {
+                self.CONFIG: p[self.CONFIG],
+                self.INPUT_VALUES: p[self.INPUT_VALUES],
+                self.DEPLOY_ACTIONS: p[self.DEPLOY_ACTIONS],
+                self.SIGNAL_TRANSPORT: p[self.SIGNAL_TRANSPORT],
+                self.NAME: p[self.NAME],
+                self.INPUT_KEY: p[self.INPUT_KEY],
+                self.INPUT_VALUES_VALIDATE: p[self.INPUT_VALUES_VALIDATE],
+            }
+        }
 
 
 def resource_mapping():
     return {
         'OS::Heat::StructuredConfig': StructuredConfig,
         'OS::Heat::StructuredDeployment': StructuredDeployment,
-        'OS::Heat::StructuredDeploymentGroup': StructuredDeploymentGroup,
         'OS::Heat::StructuredDeployments': StructuredDeployments,
     }
