@@ -116,9 +116,20 @@ class ThreadGroupManager(object):
         """
         if stack_id not in self.groups:
             self.groups[stack_id] = threadgroup.ThreadGroup()
-        return self.groups[stack_id].add_thread(self._start_with_trace,
-                                                self._serialize_profile_info(),
-                                                func, *args, **kwargs)
+
+        def log_exceptions(gt):
+            try:
+                gt.wait()
+            except Exception:
+                LOG.exception(_LE('Unhandled error in asynchronous task'))
+            except BaseException:
+                pass
+
+        th = self.groups[stack_id].add_thread(self._start_with_trace,
+                                              self._serialize_profile_info(),
+                                              func, *args, **kwargs)
+        th.link(log_exceptions)
+        return th
 
     def start_with_lock(self, cnxt, stack, engine_id, func, *args, **kwargs):
         """
@@ -978,8 +989,6 @@ class EngineService(service.Service):
         """
         try:
             resource_class = resources.global_env().get_class(type_name)
-        except exception.StackValidationFailed:
-            raise exception.ResourceTypeNotFound(type_name=type_name)
         except exception.NotFound as ex:
             raise exception.StackValidationFailed(message=ex.message)
 
@@ -1010,8 +1019,6 @@ class EngineService(service.Service):
         try:
             return resources.global_env().get_class(
                 type_name).resource_to_template(type_name)
-        except exception.StackValidationFailed:
-            raise exception.ResourceTypeNotFound(type_name=type_name)
         except exception.NotFound as ex:
             raise exception.StackValidationFailed(message=ex.message)
 
@@ -1586,30 +1593,50 @@ class EngineService(service.Service):
 
     def reset_stack_status(self):
         cnxt = context.get_admin_context()
-        filters = {'status': parser.Stack.IN_PROGRESS}
+        filters = {
+            'status': parser.Stack.IN_PROGRESS,
+            'convergence': False
+        }
         stacks = stack_object.Stack.get_all(cnxt,
                                             filters=filters,
                                             tenant_safe=False,
                                             show_nested=True) or []
         for s in stacks:
+            stack_id = s.id
             stk = parser.Stack.load(cnxt, stack=s,
                                     use_stored_context=True)
             lock = stack_lock.StackLock(cnxt, stk, self.engine_id)
-            # If stacklock is released, means stack status may changed.
             engine_id = lock.get_engine_id()
-            if not engine_id:
-                continue
-            # Try to steal the lock and set status to failed.
             try:
-                lock.acquire(retry=False)
+                with lock.thread_lock(stack_id, retry=False):
+
+                    # refetch stack and confirm it is still IN_PROGRESS
+                    s = stack_object.Stack.get_by_id(
+                        cnxt,
+                        stack_id,
+                        tenant_safe=False,
+                        eager_load=True)
+                    if s.status != parser.Stack.IN_PROGRESS:
+                        lock.release()
+                        continue
+
+                stk = parser.Stack.load(cnxt, stack=s,
+                                        use_stored_context=True)
+                LOG.info(_LI('Engine %(engine)s went down when stack '
+                             '%(stack_id)s was in action %(action)s'),
+                         {'engine': engine_id, 'action': stk.action,
+                          'stack_id': stk.id})
+
+                # Set stack and resources status to FAILED in sub thread
+                self.thread_group_mgr.start_with_acquired_lock(
+                    stk,
+                    lock,
+                    self.set_stack_and_resource_to_failed,
+                    stk
+                )
             except exception.ActionInProgress:
                 continue
-            LOG.info(_LI('Engine %(engine)s went down when stack %(stack_id)s'
-                         ' was in action %(action)s'),
-                     {'engine': engine_id, 'action': stk.action,
-                      'stack_id': stk.id})
-
-            # Set stack and resources status to FAILED in sub thread
-            self.thread_group_mgr.start_with_acquired_lock(
-                stk, lock, self.set_stack_and_resource_to_failed, stk
-            )
+            except Exception:
+                LOG.exception(_LE('Error while resetting stack: %s')
+                              % stack_id)
+                continue
