@@ -21,6 +21,7 @@ import warnings
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
+from oslo_utils import excutils
 from osprofiler import profiler
 import six
 
@@ -62,6 +63,27 @@ class ForcedCancel(BaseException):
         return "Operation cancelled"
 
 
+def reset_state_on_error(func):
+    @six.wraps(func)
+    def handle_exceptions(stack, *args, **kwargs):
+        errmsg = None
+        try:
+            return func(stack, *args, **kwargs)
+        except BaseException as exc:
+            with excutils.save_and_reraise_exception():
+                errmsg = six.text_type(exc)
+                LOG.error(_LE('Unexpected exception in %(func)s: %(msg)s'),
+                          {'func': func.__name__, 'msg': errmsg})
+        finally:
+            if stack.status == stack.IN_PROGRESS:
+                msg = _("Unexpected returning while IN_PROGRESS.")
+                stack.state_set(stack.action, stack.FAILED,
+                                errmsg if errmsg is not None else msg)
+                assert errmsg is not None, "Returned while IN_PROGRESS."
+
+    return handle_exceptions
+
+
 class Stack(collections.Mapping):
 
     ACTIONS = (
@@ -693,6 +715,7 @@ class Stack(collections.Mapping):
                 r._store()
 
     @profiler.trace('Stack.create', hide_args=False)
+    @reset_state_on_error
     def create(self):
         '''
         Create the stack and all of the resources.
@@ -780,6 +803,7 @@ class Stack(collections.Mapping):
                                            (self.status == self.FAILED))
 
     @profiler.trace('Stack.check', hide_args=False)
+    @reset_state_on_error
     def check(self):
         self.updated_time = datetime.datetime.utcnow()
         checker = scheduler.TaskRunner(self.stack_task, self.CHECK,
@@ -829,6 +853,7 @@ class Stack(collections.Mapping):
             return None
 
     @profiler.trace('Stack.adopt', hide_args=False)
+    @reset_state_on_error
     def adopt(self):
         '''
         Adopt a stack (create stack with all the existing resources).
@@ -849,6 +874,7 @@ class Stack(collections.Mapping):
         creator(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.update', hide_args=False)
+    @reset_state_on_error
     def update(self, newstack, event=None):
         '''
         Compare the current stack with newstack,
@@ -980,6 +1006,8 @@ class Stack(collections.Mapping):
             existing_params.load(newstack.t.env.user_env_as_dict())
             self.t.env = existing_params
             self.t.store(self.context)
+            backup_stack.t.env = existing_params
+            backup_stack.t.store(self.context)
         self.store()
         lifecycle_plugin_utils.do_post_ops(self.context, self,
                                            newstack, action,
@@ -1105,6 +1133,7 @@ class Stack(collections.Mapping):
         return stack_status, reason
 
     @profiler.trace('Stack.delete', hide_args=False)
+    @reset_state_on_error
     def delete(self, action=DELETE, backup=False, abandon=False):
         '''
         Delete all of the resources, and then the stack itself.
@@ -1192,6 +1221,7 @@ class Stack(collections.Mapping):
             self.id = None
 
     @profiler.trace('Stack.suspend', hide_args=False)
+    @reset_state_on_error
     def suspend(self):
         '''
         Suspend the stack, which invokes handle_suspend for all stack resources
@@ -1213,6 +1243,7 @@ class Stack(collections.Mapping):
         sus_task(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.resume', hide_args=False)
+    @reset_state_on_error
     def resume(self):
         '''
         Resume the stack, which invokes handle_resume for all stack resources
@@ -1234,6 +1265,7 @@ class Stack(collections.Mapping):
         sus_task(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.snapshot', hide_args=False)
+    @reset_state_on_error
     def snapshot(self):
         '''Snapshot the stack, invoking handle_snapshot on all resources.'''
         self.updated_time = datetime.datetime.utcnow()
@@ -1252,6 +1284,7 @@ class Stack(collections.Mapping):
                 scheduler.TaskRunner(rsrc.delete_snapshot, data)()
 
     @profiler.trace('Stack.restore', hide_args=False)
+    @reset_state_on_error
     def restore(self, snapshot):
         '''
         Restore the given snapshot, invoking handle_restore on all resources.
