@@ -19,6 +19,9 @@ wrong the tests might raise AssertionError. I've indicated in comments the
 places where actual behavior differs from the spec.
 """
 
+from keystoneclient import auth
+from keystoneclient import session
+
 from heat.common import context
 
 
@@ -31,11 +34,11 @@ class FakeClient(object):
         expected = (method, url)
         called = self.client.callstack[pos][0:2]
 
-        assert self.client.callstack, \
-            "Expected %s %s but no calls were made." % expected
+        assert self.client.callstack, ("Expected %s %s "
+                                       "but no calls were made." % expected)
 
-        assert expected == called, 'Expected %s %s; got %s %s' % \
-            (expected + called)
+        assert expected == called, 'Expected %s %s; got %s %s' % (
+            expected + called)
 
         if body is not None:
             assert self.client.callstack[pos][2] == body
@@ -46,8 +49,8 @@ class FakeClient(object):
         """
         expected = (method, url)
 
-        assert self.client.callstack, \
-            "Expected %s %s but no calls were made." % expected
+        assert self.client.callstack, ("Expected %s %s but no calls "
+                                       "were made." % expected)
 
         found = False
         for entry in self.client.callstack:
@@ -55,8 +58,8 @@ class FakeClient(object):
                 found = True
                 break
 
-        assert found, 'Expected %s %s; got %s' % \
-            (expected, self.client.callstack)
+        assert found, 'Expected %s %s; got %s' % (expected,
+                                                  self.client.callstack)
         if body is not None:
             try:
                 assert entry[2] == body
@@ -75,16 +78,37 @@ class FakeClient(object):
         pass
 
 
+class FakeAuth(auth.BaseAuthPlugin):
+
+    def __init__(self, auth_token='abcd1234', only_services=None):
+        self.auth_token = auth_token
+        self.only_services = only_services
+
+    def get_token(self, session, **kwargs):
+        return self.auth_token
+
+    def get_endpoint(self, session, service_type=None, **kwargs):
+        if (self.only_services is not None and
+                service_type not in self.only_services):
+            return None
+
+        return 'http://example.com:1234/v1'
+
+
 class FakeKeystoneClient(object):
-    def __init__(self, username='test_user', password='apassword',
+    def __init__(self, username='test_username', password='password',
                  user_id='1234', access='4567', secret='8901',
-                 credential_id='abcdxyz', auth_token='abcd1234'):
+                 credential_id='abcdxyz', auth_token='abcd1234',
+                 context=None):
         self.username = username
         self.password = password
         self.user_id = user_id
         self.access = access
         self.secret = secret
+        self.session = session.Session()
         self.credential_id = credential_id
+        self.token = auth_token
+        self.context = context
 
         class FakeCred(object):
             id = self.credential_id
@@ -92,8 +116,6 @@ class FakeKeystoneClient(object):
             secret = self.secret
         self.creds = FakeCred()
 
-        self.auth_token = auth_token
-
     def create_stack_user(self, username, password=''):
         self.username = username
         return self.user_id
@@ -127,9 +149,6 @@ class FakeKeystoneClient(object):
     def disable_stack_user(self, user_id):
         pass
 
-    def url_for(self, **kwargs):
-        return 'http://example.com:1234/v1'
-
     def create_trust_context(self):
         return context.RequestContext(username=self.username,
                                       password=self.password,
@@ -167,3 +186,10 @@ class FakeKeystoneClient(object):
 
     def stack_domain_user_token(self, user_id, project_id, password):
         return 'adomainusertoken'
+
+    @property
+    def auth_token(self):
+        if self.context is not None:
+            return self.context.auth_plugin.get_token(self.session)
+        else:
+            return self.token
