@@ -15,24 +15,42 @@ from ceilometerclient import exc as ceil_exc
 from ceilometerclient.openstack.common.apiclient import exceptions as c_a_exc
 from cinderclient import exceptions as cinder_exc
 from glanceclient import exc as glance_exc
-from heatclient import exc as heat_exc
-from keystoneclient import exceptions as keystone_exc
-from neutronclient.common import exceptions as neutron_exc
-from swiftclient import exceptions as swift_exc
-from troveclient.client import exceptions as trove_exc
-
 from heatclient import client as heatclient
+from heatclient import exc as heat_exc
+from keystoneclient.auth.identity import v3
+from keystoneclient import exceptions as keystone_exc
 import mock
-from oslo.config import cfg
-from testtools.testcase import skip
+from neutronclient.common import exceptions as neutron_exc
+from oslo_config import cfg
+from saharaclient.api import base as sahara_base
+import six
+from swiftclient import exceptions as swift_exc
+from testtools import testcase
+from troveclient import client as troveclient
 
+from heat.common import context
+from heat.common import exception
 from heat.engine import clients
 from heat.engine.clients import client_plugin
-from heat.tests.common import HeatTestCase
-from heat.tests.v1_1 import fakes
+from heat.tests import common
+from heat.tests import fakes
+from heat.tests.nova import fakes as fakes_nova
+from heat.tests import utils
 
 
-class ClientsTest(HeatTestCase):
+class ClientsTest(common.HeatTestCase):
+
+    def test_bad_cloud_backend(self):
+        con = mock.Mock()
+        cfg.CONF.set_override('cloud_backend', 'some.weird.object')
+        exc = self.assertRaises(exception.Invalid, clients.Clients, con)
+        self.assertIn('Invalid cloud_backend setting in heat.conf detected',
+                      six.text_type(exc))
+
+        cfg.CONF.set_override('cloud_backend', 'heat.engine.clients.Clients')
+        exc = self.assertRaises(exception.Invalid, clients.Clients, con)
+        self.assertIn('Invalid cloud_backend setting in heat.conf detected',
+                      six.text_type(exc))
 
     def test_clients_get_heat_url(self):
         con = mock.Mock()
@@ -72,11 +90,12 @@ class ClientsTest(HeatTestCase):
 
     @mock.patch.object(heatclient, 'Client')
     def test_clients_heat_no_auth_token(self, mock_call):
-        self.stub_keystoneclient(auth_token='anewtoken')
         con = mock.Mock()
         con.auth_url = "http://auth.example.com:5000/v2.0"
         con.tenant_id = "b363706f891f48019483f8bd6503c54b"
         con.auth_token = None
+        con.auth_plugin = fakes.FakeAuth(auth_token='anewtoken')
+        self.stub_keystoneclient(context=con)
         c = clients.Clients(con)
         con.clients = c
 
@@ -87,11 +106,12 @@ class ClientsTest(HeatTestCase):
 
     @mock.patch.object(heatclient, 'Client')
     def test_clients_heat_cached(self, mock_call):
-        self.stub_keystoneclient()
+        self.stub_auth()
         con = mock.Mock()
         con.auth_url = "http://auth.example.com:5000/v2.0"
         con.tenant_id = "b363706f891f48019483f8bd6503c54b"
         con.auth_token = "3bcc3d3a03f44e3d8377f9247b0ad155"
+        con.trust_id = None
         c = clients.Clients(con)
         con.clients = c
 
@@ -105,22 +125,6 @@ class ClientsTest(HeatTestCase):
         heat_cached = obj.client()
         self.assertEqual(heat, heat_cached)
 
-    def test_clients_auth_token_update(self):
-        fkc = self.stub_keystoneclient(auth_token='token1')
-        con = mock.Mock()
-        con.auth_url = "http://auth.example.com:5000/v2.0"
-        con.trust_id = "b363706f891f48019483f8bd6503c54b"
-        con.username = 'heat'
-        con.password = 'verysecret'
-        con.auth_token = None
-        obj = clients.Clients(con)
-        con.clients = obj
-
-        self.assertIsNotNone(obj.client('heat'))
-        self.assertEqual('token1', obj.auth_token)
-        fkc.auth_token = 'token2'
-        self.assertEqual('token2', obj.auth_token)
-
 
 class FooClientsPlugin(client_plugin.ClientPlugin):
 
@@ -128,7 +132,7 @@ class FooClientsPlugin(client_plugin.ClientPlugin):
         pass
 
 
-class ClientPluginTest(HeatTestCase):
+class ClientPluginTest(common.HeatTestCase):
 
     def test_get_client_option(self):
         con = mock.Mock()
@@ -162,37 +166,69 @@ class ClientPluginTest(HeatTestCase):
     def test_auth_token(self):
         con = mock.Mock()
         con.auth_token = "1234"
+        con.trust_id = None
 
         c = clients.Clients(con)
         con.clients = c
 
-        c.client = mock.Mock(name="client")
-        mock_keystone = mock.Mock()
-        c.client.return_value = mock_keystone
-        mock_keystone.auth_token = '5678'
+        con.auth_plugin = mock.Mock(name="auth_plugin")
+        con.auth_plugin.get_token = mock.Mock(name="get_token")
+        con.auth_plugin.get_token.return_value = '5678'
         plugin = FooClientsPlugin(con)
 
-        # assert token is from keystone rather than context
+        # assert token is from plugin rather than context
         # even though both are set
         self.assertEqual('5678', plugin.auth_token)
-        c.client.assert_called_with('keystone')
 
     def test_url_for(self):
         con = mock.Mock()
         con.auth_token = "1234"
+        con.trust_id = None
 
         c = clients.Clients(con)
         con.clients = c
 
-        c.client = mock.Mock(name="client")
-        mock_keystone = mock.Mock()
-        c.client.return_value = mock_keystone
-        mock_keystone.url_for.return_value = 'http://192.0.2.1/foo'
+        con.auth_plugin = mock.Mock(name="auth_plugin")
+        con.auth_plugin.get_endpoint = mock.Mock(name="get_endpoint")
+        con.auth_plugin.get_endpoint.return_value = 'http://192.0.2.1/foo'
         plugin = FooClientsPlugin(con)
 
         self.assertEqual('http://192.0.2.1/foo',
                          plugin.url_for(service_type='foo'))
-        c.client.assert_called_with('keystone')
+        self.assertTrue(con.auth_plugin.get_endpoint.called)
+
+    @mock.patch.object(context, "RequestContext")
+    @mock.patch.object(v3, "Token", name="v3_token")
+    def test_get_missing_service_catalog(self, mock_v3, mkreqctx):
+        class FakeKeystone(fakes.FakeKeystoneClient):
+            def __init__(self):
+                super(FakeKeystone, self).__init__()
+                self.client = self
+                self.version = 'v3'
+
+        self.stub_keystoneclient(fake_client=FakeKeystone())
+        con = mock.Mock(auth_token="1234", trust_id=None)
+        c = clients.Clients(con)
+        con.clients = c
+
+        con.auth_plugin = mock.Mock(name="auth_plugin")
+        get_endpoint_side_effects = [
+            keystone_exc.EmptyCatalog(), None, 'http://192.0.2.1/bar']
+        con.auth_plugin.get_endpoint = mock.Mock(
+            name="get_endpoint", side_effect=get_endpoint_side_effects)
+
+        mock_token_obj = mock.Mock()
+        mock_token_obj.get_auth_ref.return_value = {'catalog': 'foo'}
+        mock_v3.return_value = mock_token_obj
+
+        plugin = FooClientsPlugin(con)
+        mock_cxt_dict = {'auth_token_info': {'token': {'catalog': 'baz'}}}
+        plugin.context.to_dict.return_value = mock_cxt_dict
+
+        mkreqctx.from_dict.return_value.auth_plugin = con.auth_plugin
+
+        self.assertEqual('http://192.0.2.1/bar',
+                         plugin.url_for(service_type='bar'))
 
     def test_abstract_create(self):
         con = mock.Mock()
@@ -202,9 +238,9 @@ class ClientPluginTest(HeatTestCase):
         self.assertRaises(TypeError, client_plugin.ClientPlugin, c)
 
 
-class TestClientPluginsInitialise(HeatTestCase):
+class TestClientPluginsInitialise(common.HeatTestCase):
 
-    @skip('skipped until keystone can read context auth_ref')
+    @testcase.skip('skipped until keystone can read context auth_ref')
     def test_create_all_clients(self):
         con = mock.Mock()
         con.auth_url = "http://auth.example.com:5000/v2.0"
@@ -234,13 +270,14 @@ class TestClientPluginsInitialise(HeatTestCase):
             self.assertTrue(clients.has_client(plugin_name))
 
 
-class TestIsNotFound(HeatTestCase):
+class TestIsNotFound(common.HeatTestCase):
 
     scenarios = [
         ('ceilometer_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='ceilometer',
             exception=lambda: ceil_exc.HTTPNotFound(details='gone'),
         )),
@@ -248,6 +285,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='ceilometer',
             exception=lambda: c_a_exc.NotFound(details='gone'),
         )),
@@ -255,6 +293,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='ceilometer',
             exception=lambda: Exception()
         )),
@@ -262,13 +301,23 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='ceilometer',
             exception=lambda: ceil_exc.HTTPOverLimit(details='over'),
         )),
+        ('ceilometer_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='ceilometer',
+            exception=lambda: ceil_exc.HTTPConflict(),
+        )),
         ('cinder_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='cinder',
             exception=lambda: cinder_exc.NotFound(code=404),
         )),
@@ -276,6 +325,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='cinder',
             exception=lambda: Exception()
         )),
@@ -283,13 +333,23 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='cinder',
             exception=lambda: cinder_exc.OverLimit(code=413),
         )),
+        ('cinder_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='cinder',
+            exception=lambda: cinder_exc.ClientException(code=409),
+        )),
         ('glance_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='glance',
             exception=lambda: glance_exc.HTTPNotFound(details='gone'),
         )),
@@ -297,6 +357,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='glance',
             exception=lambda: Exception()
         )),
@@ -304,13 +365,23 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='glance',
             exception=lambda: glance_exc.HTTPOverLimit(details='over'),
         )),
+        ('glance_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='glance',
+            exception=lambda: glance_exc.HTTPConflict(),
+        )),
         ('heat_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='heat',
             exception=lambda: heat_exc.HTTPNotFound(message='gone'),
         )),
@@ -318,6 +389,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='heat',
             exception=lambda: Exception()
         )),
@@ -325,13 +397,23 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='heat',
             exception=lambda: heat_exc.HTTPOverLimit(message='over'),
         )),
+        ('heat_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='heat',
+            exception=lambda: heat_exc.HTTPConflict(),
+        )),
         ('keystone_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='keystone',
             exception=lambda: keystone_exc.NotFound(details='gone'),
         )),
@@ -339,6 +421,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='keystone',
             exception=lambda: Exception()
         )),
@@ -346,14 +429,25 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='keystone',
             exception=lambda: keystone_exc.RequestEntityTooLarge(
                 details='over'),
         )),
+        ('keystone_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='keystone',
+            exception=lambda: keystone_exc.Conflict(
+                message='Conflict'),
+        )),
         ('neutron_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: neutron_exc.NotFound,
         )),
@@ -361,6 +455,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: neutron_exc.NetworkNotFoundClient(),
         )),
@@ -368,6 +463,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: neutron_exc.PortNotFoundClient(),
         )),
@@ -375,6 +471,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: neutron_exc.NeutronClientException(
                 status_code=404),
@@ -383,6 +480,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: Exception()
         )),
@@ -390,22 +488,33 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='neutron',
             exception=lambda: neutron_exc.NeutronClientException(
                 status_code=413),
         )),
+        ('neutron_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='neutron',
+            exception=lambda: neutron_exc.Conflict(),
+        )),
         ('nova_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             is_unprocessable_entity=False,
             plugin='nova',
-            exception=lambda: fakes.fake_exception(),
+            exception=lambda: fakes_nova.fake_exception(),
         )),
         ('nova_exception', dict(
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             is_unprocessable_entity=False,
             plugin='nova',
             exception=lambda: Exception()
@@ -414,22 +523,34 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             is_unprocessable_entity=False,
             plugin='nova',
-            exception=lambda: fakes.fake_exception(413),
+            exception=lambda: fakes_nova.fake_exception(413),
         )),
         ('nova_unprocessable_entity', dict(
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             is_unprocessable_entity=True,
             plugin='nova',
-            exception=lambda: fakes.fake_exception(422),
+            exception=lambda: fakes_nova.fake_exception(422),
+        )),
+        ('nova_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            is_unprocessable_entity=False,
+            plugin='nova',
+            exception=lambda: fakes_nova.fake_exception(409),
         )),
         ('swift_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='swift',
             exception=lambda: swift_exc.ClientException(
                 msg='gone', http_status=404),
@@ -438,6 +559,7 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='swift',
             exception=lambda: Exception()
         )),
@@ -445,21 +567,33 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='swift',
             exception=lambda: swift_exc.ClientException(
                 msg='ouch', http_status=413),
         )),
+        ('swift_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='swift',
+            exception=lambda: swift_exc.ClientException(
+                msg='conflict', http_status=409),
+        )),
         ('trove_not_found', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
+            is_conflict=False,
             plugin='trove',
-            exception=lambda: trove_exc.NotFound(message='gone'),
+            exception=lambda: troveclient.exceptions.NotFound(message='gone'),
         )),
         ('trove_exception', dict(
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=False,
+            is_conflict=False,
             plugin='trove',
             exception=lambda: Exception()
         )),
@@ -467,10 +601,55 @@ class TestIsNotFound(HeatTestCase):
             is_not_found=False,
             is_over_limit=True,
             is_client_exception=True,
+            is_conflict=False,
             plugin='trove',
-            exception=lambda: trove_exc.RequestEntityTooLarge(
+            exception=lambda: troveclient.exceptions.RequestEntityTooLarge(
                 message='over'),
         )),
+        ('trove_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='trove',
+            exception=lambda: troveclient.exceptions.Conflict(
+                message='Conflict'),
+        )),
+        ('sahara_not_found', dict(
+            is_not_found=True,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=False,
+            plugin='sahara',
+            exception=lambda: sahara_base.APIException(
+                error_message='gone1', error_code=404),
+        )),
+        ('sahara_exception', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=False,
+            is_conflict=False,
+            plugin='sahara',
+            exception=lambda: Exception()
+        )),
+        ('sahara_overlimit', dict(
+            is_not_found=False,
+            is_over_limit=True,
+            is_client_exception=True,
+            is_conflict=False,
+            plugin='sahara',
+            exception=lambda: sahara_base.APIException(
+                error_message='over1', error_code=413),
+        )),
+        ('sahara_conflict', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='sahara',
+            exception=lambda: sahara_base.APIException(
+                error_message='conflict1', error_code=409),
+        )),
     ]
 
     def test_is_not_found(self):
@@ -499,6 +678,22 @@ class TestIsNotFound(HeatTestCase):
                                   client_plugin.ignore_not_found,
                                   e)
 
+    def test_ignore_conflict_and_not_found(self):
+        con = mock.Mock()
+        c = clients.Clients(con)
+        client_plugin = c.client_plugin(self.plugin)
+        try:
+            exp = self.exception()
+            exp_class = exp.__class__
+            raise exp
+        except Exception as e:
+            if self.is_conflict or self.is_not_found:
+                client_plugin.ignore_conflict_and_not_found(e)
+            else:
+                self.assertRaises(exp_class,
+                                  client_plugin.ignore_conflict_and_not_found,
+                                  e)
+
     def test_is_over_limit(self):
         con = mock.Mock()
         c = clients.Clients(con)
@@ -521,6 +716,16 @@ class TestIsNotFound(HeatTestCase):
             if ice != actual:
                 raise
 
+    def test_is_conflict(self):
+        con = mock.Mock()
+        c = clients.Clients(con)
+        client_plugin = c.client_plugin(self.plugin)
+        try:
+            raise self.exception()
+        except Exception as e:
+            if self.is_conflict != client_plugin.is_conflict(e):
+                raise
+
     def test_is_unprocessable_entity(self):
         con = mock.Mock()
         c = clients.Clients(con)
@@ -533,3 +738,24 @@ class TestIsNotFound(HeatTestCase):
                 iue = self.is_unprocessable_entity
                 if iue != client_plugin.is_unprocessable_entity(e):
                     raise
+
+
+class ClientAPIVersionTest(common.HeatTestCase):
+
+    def test_cinder_api_v1_and_v2(self):
+        self.stub_auth()
+        ctx = utils.dummy_context()
+        client = clients.Clients(ctx).client('cinder')
+        self.assertEqual(2, client.volume_api_version)
+
+    def test_cinder_api_v1_only(self):
+        self.stub_auth(only_services=['volume'])
+        ctx = utils.dummy_context()
+        client = clients.Clients(ctx).client('cinder')
+        self.assertEqual(1, client.volume_api_version)
+
+    def test_cinder_api_v2_only(self):
+        self.stub_auth(only_services=['volumev2'])
+        ctx = utils.dummy_context()
+        client = clients.Clients(ctx).client('cinder')
+        self.assertEqual(2, client.volume_api_version)
