@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,10 +11,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from mock import patch
+import mock
 
 from heat.common import exception
 from heat.common import template_format
+from heat.engine.resources.openstack.heat import cloud_watch
 from heat.engine import scheduler
 from heat.engine import watchrule
 from heat.tests import common
@@ -42,7 +43,6 @@ class CloudWatchAlarmTest(common.HeatTestCase):
 
     def setUp(self):
         super(CloudWatchAlarmTest, self).setUp()
-        utils.setup_dummy_db()
         self.ctx = utils.dummy_context()
 
     def parse_stack(self):
@@ -50,27 +50,22 @@ class CloudWatchAlarmTest(common.HeatTestCase):
         self.stack = utils.parse_stack(t)
         return self.stack
 
-    @utils.stack_delete_after
     def test_resource_create_good(self):
         s = self.parse_stack()
         self.assertIsNone(scheduler.TaskRunner(s['test_me'].create)())
 
-    @utils.stack_delete_after
     def test_resource_create_failed(self):
         s = self.parse_stack()
-        with patch.object(watchrule.WatchRule, 'store') as bad_store:
+        with mock.patch.object(watchrule.WatchRule, 'store') as bad_store:
             bad_store.side_effect = KeyError('any random failure')
             task_func = scheduler.TaskRunner(s['test_me'].create)
             self.assertRaises(exception.ResourceFailure, task_func)
 
-    @utils.stack_delete_after
     def test_resource_delete_good(self):
         s = self.parse_stack()
         self.assertIsNone(scheduler.TaskRunner(s['test_me'].create)())
         self.assertIsNone(scheduler.TaskRunner(s['test_me'].delete)())
 
-    @utils.stack_delete_after
-    @utils.wr_delete_after
     def test_resource_delete_notfound(self):
         # if a resource is not found, handle_delete() should not raise
         # an exception.
@@ -80,7 +75,29 @@ class CloudWatchAlarmTest(common.HeatTestCase):
         self.wr = watchrule.WatchRule.load(self.ctx,
                                            watch_name=res_name)
 
-        with patch.object(watchrule.WatchRule, 'destroy') as bad_destroy:
+        with mock.patch.object(watchrule.WatchRule, 'destroy') as bad_destroy:
             watch_exc = exception.WatchRuleNotFound(watch_name='test')
             bad_destroy.side_effect = watch_exc
             self.assertIsNone(scheduler.TaskRunner(s['test_me'].delete)())
+
+    def _get_watch_rule(self):
+        stack = self.parse_stack()
+        return stack['test_me']
+
+    @mock.patch.object(cloud_watch.watchrule.WatchRule, 'load')
+    def test_check(self, mock_lock):
+        res = self._get_watch_rule()
+
+        scheduler.TaskRunner(res.check)()
+        self.assertEqual((res.CHECK, res.COMPLETE), res.state)
+
+    @mock.patch.object(cloud_watch.watchrule.WatchRule, 'load')
+    def test_check_fail(self, mock_load):
+        res = self._get_watch_rule()
+        exc = cloud_watch.exception.WatchRuleNotFound(watch_name='Boom')
+        mock_load.side_effect = exc
+
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(res.check))
+        self.assertEqual((res.CHECK, res.FAILED), res.state)
+        self.assertIn('Boom', res.status_reason)
