@@ -14,13 +14,10 @@
 
 # This script is executed inside post_test_hook function in devstack gate.
 
-set -ex
+set -x
 
 export DEST=${DEST:-/opt/stack/new}
+source /opt/stack/new/devstack/accrc/admin/admin
 sudo -E $DEST/heat/heat_integrationtests/prepare_test_env.sh
-sudo -E $DEST/heat/heat_integrationtests/prepare_test_network.sh
-
-cd $DEST/tempest
-sudo tox -evenv-tempest -- stestr --test-path=$DEST/heat/heat_integrationtests --top-dir=$DEST/heat --group_regex='heat_tempest_plugin\.tests\.api\.test_heat_api[._]([^_]+)' run
-
-sudo -E $DEST/heat/heat_integrationtests/cleanup_test_env.sh
+source /opt/stack/new/devstack/accrc/demo/demo
+sudo -E tox -eintegration
