@@ -17,7 +17,9 @@ from heat_integrationtests.scenario import scenario_base
 
 
 class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
-    """Testing cfn-init and cfn-signal workability."""
+    """
+    The class is responsible for testing cfn-init and cfn-signal workability
+    """
 
     def setUp(self):
         super(CfnInitIntegrationTest, self).setUp()
@@ -25,7 +27,7 @@ class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
     def check_stack(self, sid):
         # Check status of all resources
         for res in ('WaitHandle', 'SmokeSecurityGroup', 'SmokeKeys',
-                    'CfnUser', 'SmokeServer', 'SmokeServerElasticIp'):
+                    'CfnUser', 'SmokeServer', 'IPAddress'):
             self._wait_for_resource_status(
                 sid, res, 'CREATE_COMPLETE')
 
@@ -45,6 +47,9 @@ class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
             # logs to be compared
             self._log_console_output(servers=[server])
 
+        # Check stack status
+        self._wait_for_stack_status(sid, 'CREATE_COMPLETE')
+
         stack = self.client.stacks.get(sid)
 
         # This is an assert of great significance, as it means the following
@@ -57,22 +62,7 @@ class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
             self._stack_output(stack, 'WaitConditionStatus'))
         self.assertEqual('smoke test complete', wait_status['smoke_status'])
 
-        # Check EIP attributes.
-        server_floatingip_id = self._stack_output(stack,
-                                                  'ElasticIp_Id')
-        self.assertIsNotNone(server_floatingip_id)
-
-        # Fetch EIP details.
-        net_show = self.network_client.show_floatingip(
-            floatingip=server_floatingip_id)
-        floating_ip = net_show['floatingip']['floating_ip_address']
-        port_id = net_show['floatingip']['port_id']
-
-        # Ensure that EIP was assigned to server.
-        port_show = self.network_client.show_port(port=port_id)
-        self.assertEqual(server.id, port_show['port']['device_id'])
-        server_ip = self._stack_output(stack, 'SmokeServerElasticIp')
-        self.assertEqual(server_ip, floating_ip)
+        server_ip = self._stack_output(stack, 'SmokeServerIp')
 
         # Check that created server is reachable
         if not self._ping_ip_address(server_ip):
@@ -92,7 +82,8 @@ class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
                 raise e
 
     def test_server_cfn_init(self):
-        """Check cfn-init and cfn-signal availability on the created server.
+        """
+        Check cfn-init and cfn-signal availability on the created server.
 
         The alternative scenario is the following:
             1. Create a stack with a server and configured security group.
@@ -103,11 +94,11 @@ class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
                via generated keypair.
         """
         parameters = {
-            'key_name': self.keypair_name,
-            'flavor': self.conf.instance_type,
-            'image': self.conf.image_ref,
-            'timeout': self.conf.build_timeout,
-            'subnet': self.net['subnets'][0],
+            "key_name": self.keypair_name,
+            "flavor": self.conf.instance_type,
+            "image": self.conf.image_ref,
+            "timeout": self.conf.build_timeout,
+            "subnet": self.net["subnets"][0],
         }
 
         # Launch stack
